
/********************************************************************
* Description: swapkins.c
*   Trivial kinematics for 3 axis Cartesian machine
*   It is modified to allow switch joints between X and U
*   axes with a dedicated HAL pin
*
*   Derived from a work by Fred Proctor & Will Shackleford
*
* Author:
* License: GPL Version 2
* System: Linux
*
* Copyright (c) 2004 All rights reserved.
*
* Last change: 2011
********************************************************************/

#include "kinematics.h"         /* these decls */
#include "hal.h"

struct haldata_t {
  hal_bit_t *change;
} *haldata = 0;

int kinematicsForward(const double *joints,
                     EmcPose * pos,
                     const KINEMATICS_FORWARD_FLAGS * fflags,
                     KINEMATICS_INVERSE_FLAGS * iflags)
{
   static int old_change = 0;
   if(*haldata->change == 1) {    
   pos->tran.x = joints[4];
   } else {
   pos->tran.x = joints[0];
   }
   pos->tran.y = joints[1];
   pos->tran.z = joints[2];
   pos->a = joints[3];
   pos->c = joints[5];
   pos->u = joints[6];
   pos->v = joints[7];
   pos->w = joints[8];

   return 0;
}

int kinematicsInverse(const EmcPose * pos,
                     double *joints,
                     const KINEMATICS_INVERSE_FLAGS * iflags,
                     KINEMATICS_FORWARD_FLAGS * fflags)
{
   if(*haldata->change == 1) {  
   joints[4] = pos->tran.x;
   } else {
   joints[0] = pos->tran.x;
   }
   joints[1] = pos->tran.y;
   joints[2] = pos->tran.z;
   joints[3] = pos->a;
   joints[5] = pos->c;
   joints[6] = pos->u;
   joints[7] = pos->v;
   joints[8] = pos->w;

   return 0;
}

/* implemented for these kinematics as giving joints preference */
int kinematicsHome(EmcPose * world,
                  double *joint,
                  KINEMATICS_FORWARD_FLAGS * fflags,
                  KINEMATICS_INVERSE_FLAGS * iflags)
{
   *fflags = 0;
   *iflags = 0;

   return kinematicsForward(joint, world, fflags, iflags);
}

KINEMATICS_TYPE kinematicsType()
{
   return KINEMATICS_BOTH;
}

#include "rtapi.h"              /* RTAPI realtime OS API */
#include "rtapi_app.h"          /* RTAPI realtime module decls */
#include "hal.h"

EXPORT_SYMBOL(kinematicsType);
EXPORT_SYMBOL(kinematicsForward);
EXPORT_SYMBOL(kinematicsInverse);
MODULE_LICENSE("GPL");

static int comp_id;
int rtapi_app_main(void) {
  int result;
  comp_id = hal_init("swapkins");
  if(comp_id < 0)
  return comp_id;

  haldata = hal_malloc(sizeof(struct haldata_t));

  result = hal_pin_bit_new("swapkins.change", HAL_IN,
                             &(haldata->change), comp_id);
  if(result < 0) goto error;

  hal_ready(comp_id);
  return 0;

error:
  hal_exit(comp_id);
  return result;
}

void rtapi_app_exit(void) { hal_exit(comp_id); }
